# -*- coding: utf-8 -*-
"""
.. module:: party_pe.party
    :plataform: Independent
    :synopsis: Party module models
.. moduleauthor: Carlos Eduardo Sotelo Pinto <carlos.sotelo.pinto@gmail.com>
.. copyright: (c) 2017
.. organization: Tryton-PE
.. license: GPL v3.
"""

import pytz
from datetime import datetime

from trytond.model import fields
from trytond.transaction import Transaction
from trytond.pool import Pool, PoolMeta
from trytond.pyson import Eval
from trytond.model import ModelView, ModelSQL, MatchMixin, fields, \
    sequence_ordered

__all__ = [
    'Address',
    'Party',
    'PartyIdentifier',
    'DateTime'
]

_DOCUMENT_TYPE = [
    (None, ''),
    ('pe_dtn', 'DTNDSRUC'),
    ('pe_dni', 'DNI'),
    ('pe_frg', 'CARNET EXT.'),
    ('pe_vat', 'RUC'),
    ('pe_pas', 'PASAPORTE'),
    ('pe_dip', 'CEDULA DIP.'),
]

STATES = {
    'readonly': ~Eval('active'),
    }
DEPENDS = ['active']


class Address:
    __metaclass__ = PoolMeta
    __name__ = 'party.address'

    province = fields.Many2One('country.province', 'Provincia', required=True, domain=[
        ('subdivision', '=', Eval('subdivision', -1)),
        ('parent', '=', None)
    ], states=STATES, depends=['active', 'subdivision'])
    district = fields.Many2One('country.district', 'Distrito', required=True, domain=[
        ('province', '=', Eval('province', -1)),
        ('parent', '=', None)
    ], states=STATES, depends=['active', 'province'])
    ubigeo = fields.Function(fields.Char('Ubigeo'), 'get_ubigeo')

    def get_ubigeo(self, name):
        self.ubigeo =self.subdivision.ubigeo\
         + self.province.ubigeo + self.district.ubigeo
        return self.ubigeo;    
    

    @fields.depends('district', 'subdivision', 'ubigeo', 'province')
    def on_change_district(self):
        if (self.district != None and self.subdivision != None and self.province != None):
            self.ubigeo =self.subdivision.ubigeo\
         + self.province.ubigeo + self.district.ubigeo
        else:
            self.ubigeo = None

    @fields.depends('province', 'district', 'ubigeo')
    def on_change_province(self):
        if (self.district and self.district.province != self.province):
            self.district = None
        self.ubigeo = None

    @fields.depends('subdivision', 'province')
    def on_change_subdivision(self):
        if (self.province and self.province.subdivision != self.subdivision):
            self.province = None
        self.on_change_province()

    def on_change_country(self):
        super(Address, self).on_change_country()
        self.on_change_subdivision()

    def _get_address_substitutions(self):
        context = Transaction().context
        substitutions = {
            'party_name': '',
            'name': self.name or '',
            'street': self.street or '',
            'zip': self.zip or '',
            'city': self.city or '',
            'district': self.district or '',
            'province': self.province.name or '', 
            'subdivision': self.subdivision.name if self.subdivision else '',
            'subdivision_code': (self.subdivision.code.split('-', 1)[1]
                if self.subdivision else ''),
            'country': self.country.name if self.country else '',
            'country_code': self.country.code if self.country else '',
            }

        return substitutions

class Party:
    __metaclass__ = PoolMeta
    __name__ = 'party.party'

    document_type = fields.Function(
        fields.Char(u"Tipo de documento"),
        'get_document_type',
        searcher='search_document_type'
    )
    document_type_name = fields.Function(
        fields.Char(u"Tipo de documento"),
        'get_document_type_name'
    )
    document_number = fields.Function(
        fields.Char(u"Nro. de Documento"),
        'get_document_number',
        searcher='search_document_number'
    )
    invoicing_address = fields.Function(
        fields.Char(u"Direcciónn de facturación"),
        'get_invoicing_address',
    )


    @classmethod
    def _vat_types(cls):
        return ['pe_vat']

    def get_document_type(self, name):
        if len(self.identifiers) == 1:
            return self.identifiers[0].type

    def get_document_type_name(self, name):
        if len(self.identifiers) == 1 and self.identifiers[0].type in dict(_DOCUMENT_TYPE):
            return dict(_DOCUMENT_TYPE)[self.identifiers[0].type]

    def get_document_number(self, name):
        if len(self.identifiers) == 1:
            return self.identifiers[0].code

    @classmethod
    def search_document_type(cls, name, clause):
        return [
            ('identifiers.type',) + tuple(clause[1:]),
        ]

    @classmethod
    def search_document_number(cls, name, clause):
        return [
            ('identifiers.code',) + tuple(clause[1:]),
        ]

    def get_address_parts(self):
        #for address in self.invoicing_address:
        for address in self.addresses:
            if address.invoice:
                subdivision = ""
                country = ""
                province = ""
                district = ""
                if address.subdivision:
                    subdivision = address.subdivision.name
                if address.country:
                    country = address.country.name
                if address.province:
                    province = address.province.name
                if address.district:
                    district = address.district.name
                return dict(
                    street=address.street,
                    city=address.city,
                    subdivision=subdivision,
                    country=country,
                    province=province,
                    district=district
                )
        return dict()

    def get_invoicing_address(self, name):
        #for address in self.invoicing_address:
        for address in self.addresses:
            if address.invoice:
                country = ""
                subdivision = ""
                province = ""
                district = ""
                if address.subdivision:
                    subdivision = address.subdivision.name
                if address.country:
                    country = address.country.name
                if address.province:
                    province = address.province.name
                if address.district:
                    district = address.district.name
                return "{street}, {district}, {province},{subdivision}, {country},".format(
                    street=address.street,
                    district=district,
                    province=province,
                    subdivision=subdivision,
                    country=country,
                )
        return ""


class PartyIdentifier:
    'Party Identifier'
    __metaclass__ = PoolMeta
    __name__ = 'party.identifier'

    type = fields.Selection('get_types', 'Type')

    @classmethod
    def get_types(cls):
        return _DOCUMENT_TYPE


class DateTime:
    'Date Time'
    __metaclass__ = PoolMeta
    __name__ = 'ir.date'

    @classmethod
    def time(cls, timezone=None):
        """
        Get current date time according timezone

        :param timezone: string representing standar time zone

        :return: datetime with time zoned now
        """
        return cls.now(timezone).strftime("%H:%M")

    @classmethod
    def now(cls, timezone=None):
        """
        Get current date time according timezone

        :param timezone: string representing standar time zone

        :return: datetime with time zoned now
        """

        Company = Pool().get('company.company')
        company_id = Transaction().context.get('company')
        if timezone is None and company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)
        return datetime.now(timezone)

    @classmethod
    def localize(cls, utc_datetime, timezone=None):
        """
        Localize utc_datetime according timzezone

        :param utc_datetime: date without timezone, we will take this as utc
        :param timezone: string with timezone we want to localize

        :return: datetime with time zoned datetime
        """

        if utc_datetime is None:
            return None

        Company = Pool().get('company.company')
        company_id = Transaction().context.get('company')
        if timezone is None and company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)
        utc_datetime = pytz.timezone('UTC').localize(utc_datetime)
        return utc_datetime.astimezone(timezone)

    @classmethod
    def localize_str(cls, utc_datetime, timezone=None):
        """"
        Localize utc_datetime according timzezone

        :param utc_datetime: date without timezone, we will take this as utc
        :param timezone: string with timezone we want to localize

        :return: string with time zoned datetime
        """

        localize = cls.localize(utc_datetime, timezone)
        if localize is None:
            return ''

        return localize.strftime("%Y-%m-%d %H:%M")
